/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::ProsthesisObserver
 * @author     JuliaStarke ( julia dot starke at kit dot edu )
 * @date       2018
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once


//#include <ArmarXCore/core/Component.h>
//#include <RobotAPI/interface/units/ProsthesisInterface.h>
#include <RobotAPI/interface/units/ProsthesisObserverInterface.h>
#include <ArmarXCore/observers/Observer.h>
#include <RobotAPI/interface/visualization/DebugDrawerInterface.h>

namespace armarx
{
    /**
     * @class ProsthesisObserverPropertyDefinitions
     * @brief
     */
    class ProsthesisObserverPropertyDefinitions:
        public armarx::ObserverPropertyDefinitions
    {
    public:
        ProsthesisObserverPropertyDefinitions(std::string prefix):
            armarx::ObserverPropertyDefinitions(prefix)
        {
            //defineRequiredProperty<std::string>("PropertyName", "Description");
            defineOptionalProperty<std::string>("ProsthesisTopicName", "ProsthesisMotorValues", "Name of the Prosthesis Topic");
            defineOptionalProperty<std::string>("DebugDrawerTopic", "DebugDrawerUpdates", "Name of the DebugDrawerTopic");
        }
    };

    /**
     * @defgroup Component-ProsthesisObserver ProsthesisObserver
     * @ingroup RobotAPI-Components
     * A description of the component ProsthesisObserver.
     *
     * @class ProsthesisObserver
     * @ingroup Component-ProsthesisObserver
     * @brief Brief description of class ProsthesisObserver.
     *
     * Detailed description of class ProsthesisObserver.
     */
    class ProsthesisObserver :
        virtual public Observer,
        virtual public ProsthesisObserverInterface
    {
    public:
        ProsthesisObserver() {}

        /**
         * @see armarx::ManagedIceObject::getDefaultName()
         */
        std::string getDefaultName() const override
        {
            return "ProsthesisObserver";
        }

    protected:
        /**
         * @see armarx::ManagedIceObject::onInitComponent()
         */
        void onInitObserver() override;

        /**
         * @see armarx::ManagedIceObject::onConnectComponent()
         */
        void onConnectObserver() override;

        /**
         * @see armarx::ManagedIceObject::onDisconnectComponent()
         */
        //void onDisconnectObserver() override;

        /**
         * @see armarx::ManagedIceObject::onExitComponent()
         */
        void onExitObserver() override;

        /**
         * @see PropertyUser::createPropertyDefinitions()
         */
        armarx::PropertyDefinitionsPtr createPropertyDefinitions() override;

    private:

        Mutex dataMutex;
        DebugDrawerInterfacePrx debugDrawerPrx;

        // ProsthesisListenerInterface interface
    public:
        void reportMotorValues(const ProsthesisMotorValues &motorValues, const Ice::Current &) override;
    };
}
